/*
 * Problem: Flood-IT
 * Author: Alexander "espr1t" Georgiev
 * Contest: CodeIT 2011, Round 1
 * Keywords: Greedy, NP
 */

#include <cstdio>
#include <ctime>
#include <algorithm>
#include <vector>
#include <queue>
#include <cstring>
#include <map>

#define MAX 131072
#define INF 999666333

#define MAX_SIZE 360
#define MAX_MOVES 67600
#define MAX_COLORS 512

#define MAX_EXEC_TIME 0.8
#define DEBUG_OUTPUT 0
#define TIME_CHECK 100

#define INITIAL_RANDOM_CELLS 5

using namespace std;

struct NeighbourInfo
{
	int color, count;
	vector <int> who;
};

struct NodeInfo
{
	int color, count;
	vector <NeighbourInfo> v;
	NodeInfo(int color_ = 0, int count_ = 0) : color(color_), count(count_) {}
};

unsigned startTime;
int timeCounter, timeFlag;
int moves[MAX_MOVES];
int ansRow, ansCol, ansMoves;
int dir[4][2] = { {-1, 0}, {0, 1}, {1, 0}, {0, -1} };

int n, k;
int a[MAX_SIZE][MAX_SIZE];
int board[MAX_SIZE][MAX_SIZE];
int vis[MAX_SIZE][MAX_SIZE], used[MAX];

int numNodes;
NodeInfo info[MAX];

int maxDepth;
vector <int> colors;
vector <int> nodes[MAX_COLORS];
int colIdx[MAX_COLORS];


double getTime()
{
	return (double)(clock() - startTime) / (double)CLOCKS_PER_SEC;
}

int recurse(int remDepth, int remNodes, int& bestColor)
{
	if (remNodes <= 0) return -remDepth;
	if (remDepth <= 0) return +remNodes;
	
	int best = INF; bestColor = -1;
	if (++timeCounter > TIME_CHECK)
	{
		timeCounter = 0;
		if (getTime() > MAX_EXEC_TIME) {timeFlag = 1; return best;}
	}
	
	for (int ccol = 0; ccol < (int)colors.size(); ccol++)
	{
		int color = colors[ccol];
		vector <int> save = nodes[color];
		
		remNodes -= nodes[color].size();
		nodes[color].clear();
		
		swap(colors[ccol], colors[(int)colors.size() - 1]);
		colIdx[colors[ccol]] = ccol;
		colIdx[colors.back()] = -1; colors.pop_back();
		
		int remove = 0;
		map <int, int> use;
		for (int i = 0; i < (int)save.size(); i++)
		{
			int node = save[i];
			for (int c = 0; c < (int)info[node].v.size(); c++)
			{
				for (int j = 0; j < (int)info[node].v[c].who.size(); j++)
				{
					if (!used[info[node].v[c].who[j]])
					{
						if (colIdx[info[node].v[c].color] == -1)
						{
							colors.push_back(info[node].v[c].color);
							colIdx[colors.back()] = (int)colors.size() - 1;
							remove++;
						}
						use[info[node].v[c].color]++;
						used[info[node].v[c].who[j]] = 1;
						nodes[info[node].v[c].color].push_back(info[node].v[c].who[j]);
					}
				}
			}
		}
		
		int cur = INF, withColor = -1;
		cur = recurse(remDepth - 1, remNodes, withColor);
		if (cur < best) {best = cur; bestColor = color;}
		
		for (map <int, int> :: iterator it = use.begin(); it != use.end(); it++)
		{
			for (int i = 0; i < it->second; i++)
			{
				used[nodes[it->first].back()] = 0;
				nodes[it->first].pop_back();
			}
		}
		for (int i = 0; i < remove; i++)
		{
			colIdx[colors.back()] = -1;
			colors.pop_back();
		}
		
		colors.push_back(color);
		swap(colors[ccol], colors[(int)colors.size() - 1]);
		colIdx[color] = ccol; colIdx[colors.back()] = (int)colors.size() - 1;
		
		nodes[color] = save;
		remNodes += nodes[color].size();
		if (timeFlag) break;
	}
	return best;
}

void iterativeDeepening(int startRow, int startCol)
{
//	fprintf(stdout, "Running ID with startRow %d and startCol %d\n", startRow, startCol); fflush(stdout);
	timeCounter = 0; timeFlag = 0;
	for (maxDepth = 1; maxDepth <= 10; maxDepth++)
	{
		if (getTime() > MAX_EXEC_TIME) break;
		
		colors.clear();
		memset(colIdx, -1, sizeof(colIdx));
		for (int i = 0; i <= k; i++) nodes[i].clear();
		for (int i = 0; i <= numNodes; i++) used[i] = 0;

		used[board[startRow][startCol]] = 1;
		colors.push_back(info[board[startRow][startCol]].color);
		colIdx[colors.back()] = (int)colors.size() - 1;
		nodes[info[board[startRow][startCol]].color].push_back(board[startRow][startCol]);
		
		vector <int> ans;
		for (int rem = numNodes; rem > 0;)
		{
			int best = INF, with = -1;
			best = recurse(maxDepth, rem, with);
			if (getTime() > MAX_EXEC_TIME) {ans.clear(); break;}
//			fprintf(stdout, "Best is %d with color %d\n", best, with); fflush(stdout);

			ans.push_back(with);
			rem -= nodes[with].size();
			colors[colIdx[with]] = colors.back();
			colIdx[colors[colIdx[with]]] = colIdx[with];
			colors.pop_back(); colIdx[with] = -1;
			
			for (int i = 0; i < (int)nodes[with].size(); i++)
			{
				int node = nodes[with][i];
				for (int c = 0; c < (int)info[node].v.size(); c++)
				{
					for (int j = 0; j < (int)info[node].v[c].who.size(); j++)
					{
						if (!used[info[node].v[c].who[j]])
						{
							if (colIdx[info[node].v[c].color] == -1)
							{
								colors.push_back(info[node].v[c].color);
								colIdx[colors.back()] = (int)colors.size() - 1;
							}
							used[info[node].v[c].who[j]] = 1;
							nodes[info[node].v[c].color].push_back(info[node].v[c].who[j]);
						}
					}
				}
				
			}
			nodes[with].clear();
		}
		
//		fprintf(stdout, "Finished this iteration\n============================\n");
		if ((int)ans.size() > 0 && (int)ans.size() - 1 < ansMoves)
		{
			ansMoves = (int)ans.size() - 1;
			ansRow = startRow; ansCol = startCol;
			for (int i = 0; i < ansMoves; i++)
				moves[i] = ans[i + 1];
		}
	}
}


void eval(int startRow, int startCol)
{
	int count[MAX_COLORS];
	memset(used, 0, sizeof(used));
	memset(count, 0, sizeof(count));
	
	static vector <int> v[MAX_COLORS];
	used[board[startRow][startCol]] = 1;
	count[a[startRow][startCol]] = info[board[startRow][startCol]].count;
	v[a[startRow][startCol]].push_back(board[startRow][startCol]);

	vector <int> ans;
	for (int rem = n * n; rem > 0; )
	{
		int color = 1;
		for (int i = 2; i <= k; i++)
			if (count[color] < count[i]) color = i;
		ans.push_back(color);
		
		for (int i = 0; i < (int)v[color].size(); i++)
		{
			int node = v[color][i];
			for (int c = 0; c < (int)info[node].v.size(); c++)
			{
				for (int j = 0; j < (int)info[node].v[c].who.size(); j++)
				{
					if (!used[info[node].v[c].who[j]])
					{
						used[info[node].v[c].who[j]] = 1;
						count[info[node].v[c].color] += info[info[node].v[c].who[j]].count;
						v[info[node].v[c].color].push_back(info[node].v[c].who[j]);
					}
				}
			}
		}
		rem -= count[color];
		v[color].clear(); count[color] = 0;
	}
	
	if (ansMoves == -1 || ansMoves > (int)ans.size() - 1)
	{
		ansMoves = (int)ans.size() - 1;
		ansRow = startRow; ansCol = startCol;
		for (int i = 0; i < (int)ans.size() - 1; i++)
			moves[i] = ans[i + 1];
	}
}

vector < pair <int, int> > getCellQueue()
{
	// Four central
	vector < pair <int, int> > ret;
	ret.push_back(make_pair(n / 2, n / 2));
	for (int offRow = -1; offRow <= 0; offRow++)
		for (int offCol = -1; offCol <= 0; offCol++)
			if (board[n / 2][n / 2] != board[n / 2 + offRow][n / 2 + offCol])
				ret.push_back(make_pair(n / 2 + offRow, n / 2 + offCol));
	// Some random
	for (int i = 0; i < INITIAL_RANDOM_CELLS; i++)
	{
		int row = rand() % n, col = rand() % n;
		int flag = 1;
		for (int c = 0; c < (int)ret.size(); c++)
			if (board[ret[c].first][ret[c].second] == board[row][col]) {flag = 0; break;}
		if (flag) ret.push_back(make_pair(row, col));
	}
	return ret;
}

void solve()
{
	vector < pair <int, int> > cellQueue = getCellQueue();
	
	// Simple Greedy
	for (int i = 0; i < (int)cellQueue.size(); i++)
	{
		if (getTime() > MAX_EXEC_TIME) break;
		eval(cellQueue[i].first, cellQueue[i].second);
	}
	
	// Get best cell in front
	for (int i = 0; i < (int)cellQueue.size(); i++)
		if (cellQueue[i].first == ansRow && cellQueue[i].second == ansCol)
			{swap(cellQueue[i], cellQueue[0]); break;}
	
	// Iterative deepening on the best so far
	int idx = 0;
	while (1)
	{
		if (getTime() > MAX_EXEC_TIME) break;
		if (idx < (int)cellQueue.size())
		{
			iterativeDeepening(cellQueue[idx].first, cellQueue[idx].second);
			idx++;
		}
		else
		{
			int row = rand() % n, col = rand() % n;
			iterativeDeepening(row, col);
		}
	}
}

void createGraph()
{
	static int added[MAX], vv[MAX_COLORS];
	static NeighbourInfo ninfo[MAX_COLORS];
	for (int i = 0; i < MAX_COLORS; i++) ninfo[i].color = i;

	numNodes = 0;
	memset(vv, 0, sizeof(vv));
	memset(added, 0, sizeof(added));
	memset(board, -1, sizeof(board));

	// Color nodes
	for (int row = 0; row < n; row++)
	{
		for (int col = 0; col < n; col++) if (board[row][col] == -1)
		{
			info[numNodes].count = 0;
			info[numNodes].color = a[row][col];

			queue < pair <int, int> > q;
			q.push(make_pair(row, col)); board[row][col] = numNodes;
			
			while (!q.empty())
			{
				int crow = q.front().first;
				int ccol = q.front().second;
				q.pop(); info[numNodes].count++;
				
				for (int i = 0; i < 4; i++)
				{
					int nrow = crow + dir[i][0]; if (nrow < 0 || nrow >= n) continue;
					int ncol = ccol + dir[i][1]; if (ncol < 0 || ncol >= n) continue;
					if (board[nrow][ncol] == -1 && a[nrow][ncol] == info[numNodes].color)
					{
						board[nrow][ncol] = numNodes;
						q.push(make_pair(nrow, ncol));
					}
				}
			}
			numNodes++;
		}
	}
	
	memset(vis, 0, sizeof(vis));
	for (int row = 0; row < n; row++)
	{
		for (int col = 0; col < n; col++) if (!vis[row][col])
		{
			int node = board[row][col];

			vector <int> ncolors;
			queue < pair <int, int> > q;
			q.push(make_pair(row, col)); vis[row][col] = 1;
			
			while (!q.empty())
			{
				int crow = q.front().first;
				int ccol = q.front().second;
				q.pop();
				
				for (int i = 0; i < 4; i++)
				{
					int nrow = crow + dir[i][0]; if (nrow < 0 || nrow >= n) continue;
					int ncol = ccol + dir[i][1]; if (ncol < 0 || ncol >= n) continue;
					if (board[nrow][ncol] == node && !vis[nrow][ncol])
					{
						vis[nrow][ncol] = 1;
						q.push(make_pair(nrow, ncol));
					}
					else if (board[nrow][ncol] != node && !added[board[nrow][ncol]])
					{
						if (!vv[a[nrow][ncol]])
							vv[a[nrow][ncol]] = 1, ncolors.push_back(a[nrow][ncol]);

						added[board[nrow][ncol]] = 1;
						ninfo[a[nrow][ncol]].who.push_back(board[nrow][ncol]);
						ninfo[a[nrow][ncol]].count += info[board[nrow][ncol]].count;
					}
				}
			}
			
			for (int i = 0; i < (int)ncolors.size(); i++)
			{
				info[node].v.push_back(ninfo[ncolors[i]]);

				vv[ncolors[i]] = 0;
				for (int c = 0; c < (int)ninfo[ncolors[i]].who.size(); c++)
					added[ninfo[ncolors[i]].who[c]] = 0;
				ninfo[ncolors[i]].count = 0;
				ninfo[ncolors[i]].who.clear();
			}
		}
	}
}

void init()
{
	ansMoves = -1;
	ansRow = -1;
	ansCol = -1;
}

void readInput()
{
	FILE* in = fopen("flood-it.in", "rt");
	fscanf(in, "%d %d", &n, &k);
	for (int i = 0; i < n; i++)
		for (int c = 0; c < n; c++)
			fscanf(in, "%d", &a[i][c]);
	fclose(in);
}

void printOutput()
{
	FILE* out = fopen("flood-it.out", "wt");
	fprintf(out, "%d\n", ansMoves);
	fprintf(out, "%d %d\n", ansRow, ansCol);
	for (int i = 0; i < ansMoves; i++)
		fprintf(out, "%d\n", moves[i]);
	fclose(out);
}

int main(void)
{
	startTime = clock();
	init();
	readInput();
	createGraph();
	solve();
	printOutput();
	return 0;
}
